;;;
;;; Copyright (c) 2006 Jonathan Heusser <jonny@drugphish.ch>
;;;
;;; Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
;;; associated documentation files (the "Software"), to deal in the Software without restriction, 
;;; including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
;;; and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, 
;;; subject to the following conditions:
;;;
;;; The above copyright notice and this permission notice shall be included in all copies or substantial 
;;; portions of the Software.
;;;
;;; THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
;;; LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
;;; IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
;;; WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
;;; SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
;;;

(eval-when (:compile-toplevel :load-toplevel :execute)
  (asdf:operate 'asdf:load-op 'trivial-http))

(in-package :cl-clickatell)

;; change these parameters according to your setup
(defparameter *user* "")
(defparameter *pass* "")
(defparameter *base-url* "196.7.150.220")
(defparameter *api-id* 0)
(defparameter *country-prefix* "41")
(defparameter *sender* "lisp")

(define-condition send-sms-error (error)
  ((nested-error :initarg :nested-error :reader nested-error)))

(define-condition send-sms-failed (error)
  ((text :initarg :text :reader text)))

(define-condition authentication-failed (error)
  ((text :initarg :text :reader text)))

(defun config-sms (user pass api-id country-prefix sender)
  (setq *user* user *pass* pass *api-id* api-id *country-prefix* country-prefix
	*sender* sender))

(defun set-country-prefix (prefix)
  (setq *country-prefix* prefix))

(defun set-sender (sender)
  (setq *sender* sender))

(defun open-location (loc)
  "reads from the web location loc and returns a string"
  (with-output-to-string (str)
			 (with-open-stream (s (trivial-http:http-get loc t))
			   (loop
			       (let ((l (read-line s nil nil)))
				 (unless l (return))
				 (format str "~a" l) (terpri))))))

(defun urlencode (url) 
  "encodes every non-alphanumeric char except space . and -
   by its hex representation. space becomes + additionally."
  (with-output-to-string (s) 
			 (loop for char across url do
			       (if (not (char= char #\SPACE)) 
				   (if (and (not (alphanumericp char))
					    (not (char= char #\.))
					    (not (char= char #\-)))
				       (format s "%~X" (char-code char))
				       (format s "~a" char)))
			       (if (char= char #\SPACE)
				   (format s "+")))))

(defun get-session-id (&key (base *base-url*) (user *user*)
                            (pass *pass*) (api-id *api-id*))
  (let* ((baseurl (concatenate 'string "http://" base))
         (url (format nil "~A/http/auth?user=~A&password=~A&api_id=~A"
                      baseurl user pass api-id))
         (session (open-location url))
         (session-list (split-sequence:split-sequence #\: session)))
    (when (string= (car session-list) "ERR")
      (error 'authentication-failed :text (cadr session-list)))
    (list (car session-list) (remove #\SPACE (cadr session-list)))))

(defun ping-session (session-id)
  "prevents a session from expiring. Returns T when successful and signals
   AUTHENTICATION-FAILED otherwise."
  (let* ((url (format nil "http://api.clickatell.com/http/ping?session_id=~A" session-id))
         (ret (open-location url))
         (answer (split-sequence:split-sequence #\: ret)))
    (when (string= (car answer) "ERR")
      (error 'authentication-failed :text (cadr answer)))
    t))

(defun query-message (apimsg-id session-id)
  "returns the status as string of a given sms identified by the apimsg-id and session-id."
  (let* ((url (format nil "http://api.clickatell.com/http/querymsg?session_id=~A&apimsgid=~A"
                      session-id apimsg-id))
         (answer (split-sequence:split-sequence #\: (open-location url))))
    (when (string= (car answer) "ERR")
      (error 'authentication-failed :text (cadr answer)))
    (remove #\SPACE (car (last answer)))))

(defun query-balance (session-id)
  (let* ((url (format nil "http://api.clickatell.com/http/getbalance?session_id=~A"
                      session-id))
         (answer (open-location url))
         (authcheck (split-sequence:split-sequence #\: answer)))
    (when (string= (car authcheck) "ERR")
      (error 'authentication-failed :text (cadr authcheck)))
    answer))
    
(defun send-sms (to text &key (country *country-prefix*)
		    (base *base-url*) (sender *sender*) (user *user*)
		    (pass *pass*) (api-id *api-id*))	    
  "sends an sms to number TO with TEXT as body.
  an example: send-sms \"797764747\" \"lisp!\".
  Returns a list (apimsgid sessionid) when successful. Signals SEND-SMS-ERROR otherwise." 
  (handler-bind ((error (lambda (c) (error 'send-sms-error :nested-error c))))
      (let* ((baseurl (concatenate 'string "http://" base))
             (text (urlencode text))
	     (to (concatenate 'string country to))
             (session-list (get-session-id :base base :user user :pass pass :api-id api-id))
	     (url (format nil "~A/http/sendmsg?session_id=~A&to=~A&text=~A&from=~A"
			      baseurl (remove #\SPACE (cadr session-list)) to text sender))
	     (messagestring (open-location url))
             (answer (split-sequence:split-sequence #\: messagestring)))
        (if (string= (car answer) "ID")
          (list (remove #\SPACE (cadr answer)) (cadr session-list))
          (error 'send-sms-failed :text (list answer session-list to))))))
        
